/*
 * twd.c
 *
 * Created: 16.08.2017 11:35:16
 *  Author: dsosnowski
 */ 
/*
 * mod1535_ton_toff.c
 *
 * Created: 2014-05-29 12:23:11
 *  Author: dsosnowski
 */ 


#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "ee_mem.h"
#include "wdog.h"
#include "defs.h"

//0...9, 0, -, null
static char digits[15] = {0x42, 0xEE, 0x92, 0x86, 0x2E, 0x07, 0x03, 0xCE, 0x02, 0x06, 0xEC, 0xBF, 0xFF, 0xDF, 0xF7};

#define NULL_INDX	12

//-------------------------
void HardInit(void);
void Display(uint8_t val);
void DisplaySpecial(uint8_t i1, uint8_t i2);
uint8_t SwitchScan(void);
uint8_t SwitchActionGet(void);
void TimeChange(uint8_t action);
void SaveSettProc(void);
void RestoreSett(void);


struct {
	uint8_t chars[2];
	uint8_t dot[2];
} display;

struct {
	volatile uint8_t div;
	volatile uint8_t tick;
	uint8_t seconds;
	uint8_t action;
	uint8_t time;
	uint8_t step;
} system;

struct {
	uint8_t save;
	uint8_t time;
} config;


int main(void)
{
	uint8_t afteroff;
	
	HardInit();
	_delay_ms(100);
	RestoreSett();    
	while(1)
    {
	
		//--------------------------------------------------
		SwitchScan();
		
		if (SwitchActionGet() > 0){		
			//--------------------------------------------------
			if (system.step <= STEP_STOP){		
				if (SwitchActionGet() == SW3_ACTION){
					system.step = STEP_START;
				}

				if (SwitchActionGet() == SW1_ACTION){
					TimeChange(SW1_ACTION);
					config.save = SAVE_DELAY;
				}
				if (SwitchActionGet() == SW2_ACTION){
					TimeChange(SW2_ACTION);
					config.save = SAVE_DELAY;
				}
			} else {		 
			//--------------------------------------------------
			//if (system.step >= STEP_START){

				if (SwitchActionGet() == SW3_ACTION){
					if (system.step < STEP_PAUSE){
						system.step = STEP_PAUSE;
					} else {
						system.step = STEP_RUN;
					}
				}
				
				if (SwitchActionGet() == SW_HOLD_ACTION){
					system.step = STEP_STOP;
				}			
			}
		}
			
		if (system.step == STEP_START){
			TCNT1 = 0;
			OUT_SET;
			system.time = config.time;
			system.seconds = 0;
			system.step = STEP_RUN;
			system.tick = 0;
			system.div = 0;
		}

		//--------------------------------------------------
		if (system.tick > 0){
			system.tick = 0;
			if (system.step == STEP_RUN){
				if (GET_CFG == 0) {				//sek					
					if (system.time > 0) system.time--;
				} else{
					system.seconds++;
					if (system.seconds > 59){
						system.seconds = 0;
						if (system.time > 0) system.time--;
					}				
				}
				if (system.time == 0){
					system.step = STEP_STOP;
				}
			}
		}
		
		//--------------------------------------------------
		if (system.step == STEP_STOP){
			system.step = STEP_AFTERSTOP;
			afteroff = AFTERSTOP_TIME;
			OUT_OFF;
		}
		if (system.step == STEP_AFTERSTOP){
			if (afteroff > 0) {
				afteroff--;
			} else {
				system.step = STEP_IDLE;
			}
		}

		//--------------------------------------------------
		if ((system.step >= STEP_START)||(system.step == STEP_AFTERSTOP)){
			if (system.step == STEP_PAUSE){
				if (system.div == 0){
					Display(system.time);
					display.dot[1] = 1;					
				} else {
					DisplaySpecial(NULL_INDX, NULL_INDX);
					display.dot[1] = 0;					
				}
				
			} else if (system.step == STEP_AFTERSTOP){
				Display(system.time);
				display.dot[1] = 0;
				
			} else {
				Display(system.time);				
				if (system.div == 0){
					display.dot[1] = 1;
				} else {
					display.dot[1] = 0;
				}				
			}
		} else {
			Display(config.time);
			if (GET_CFG == 0){
				display.dot[1] = 0;			
			} else {
				display.dot[1] = 1;				
			}
			
		}
		
		//--------------------------------------------------
		SaveSettProc();

		//--------------------------------------------------
		_delay_ms(20);
		WdtReset();
    }
}


//-----------------------------------------------------------
void HardInit(void)
{
	UI_PORT = (1<<SW1_PIN | 1<<SW2_PIN | 1<<SW3_PIN | 1<<CFG_PIN);
	UI_DDR = (1<<ANODE1 | 1<<ANODE2 | 1<<RELAY_PIN);
	
	SEG_PORT = 0x00;
	SEG_DDR = 0xFF;

	//for display
	TCCR0B = (1<<PSR0 | 1<<CS01);
	TIMSK |= (1<<TOIE0);

	//for tick
	TCCR1A = 0;
	TCCR1B = (1<<PSR1 | 1<<CS13 | 1<<CS12 | 1<<CS11 | 1<<CS10);		//div 16384
	OCR1C = TICK_0_5S;
	TIMSK |= (1<<TOIE1);

	WdtSet(7);

	sei();
}

//-----------------------------------------------------------
//Display
ISR(TIMER0_OVF_vect)
{
	static char anode;
	if (anode < ANODES_FACTOR) anode++;
	else anode = 0;
	
	switch (anode)
	{
		case 0:
			ANODES_PORT |= (1<<ANODE2);
			SEG_PORT = display.chars[0];
			if (display.dot[0]) DP_SET;
			ANODES_PORT &= ~(1<<ANODE1);
		break;
		
		case 1:
			ANODES_PORT |= (1<<ANODE1);
			SEG_PORT = display.chars[1];
			if (display.dot[1]) DP_SET;
			ANODES_PORT &= ~(1<<ANODE2);
		break;

		case 2:
		ANODES_PORT |= (1<<ANODE2);
		//SEG_PORT = display.chars[1];
		//ANODES_PORT &= ~(1<<ANODE2);
		break;
	}
}

//-----------------------------------------------------------
//Tick
ISR(TIMER1_OVF_vect)
{
	system.div++;
	if (system.div >= 2)
	{
		system.div = 0;
		system.tick = 1;
	}
}

//-----------------------------------------------------------
void Display(uint8_t val){
	uint8_t i;
	i = (val / 10);
	if (i > 10) i = 0;
	display.chars[0] = digits[i];
	
	i = (val % 10);
	display.chars[1] = digits[i];
}

//-----------------------------------------------------------
void DisplaySpecial(uint8_t i1, uint8_t i2){
	display.chars[0] = digits[i1];
	display.chars[1] = digits[i2];	
}

//-----------------------------------------------------------
uint8_t SwitchScan(void){
	static uint8_t prev;
	static uint8_t input_counter;
	uint8_t result = 0;
	
	if ((UI_PIN & (1<<SW1_PIN)) == 0) result = SW2_ACTION;
	if ((UI_PIN & (1<<SW2_PIN)) == 0) result = SW1_ACTION;
	if ((UI_PIN & (1<<SW3_PIN)) == 0) result = SW3_ACTION;
	
	if (result > 0){
		if (result != prev)	input_counter = 0;
		if (input_counter < 0xFF) input_counter++;
		
		prev = result;
		result = 0;			
		
		if (input_counter == SWITCH_HOLD){
			if (prev == SW3_ACTION){
				result = SW_HOLD_ACTION;
			} else {
				result = prev;
				input_counter = (SWITCH_HOLD - 2);
			}
		}
	} else {
		if ((input_counter > SWITCH_PRESS) && (input_counter < SWITCH_HOLD)){
			result = prev;
		}
		prev = 0;
		input_counter = 0;
	}
	system.action = result;
	return result;
}

//-----------------------------------------------------------
uint8_t SwitchActionGet(void){
	return system.action;
}

//-----------------------------------------------------------
void TimeChange(uint8_t action){
	if (action == SW1_ACTION){
		if (config.time < TIME_MAX){
			config.time++;
		}
	}
	if (action == SW2_ACTION){
		if (config.time > 0){
			config.time--;
		}
	}
}

//-----------------------------------------------------------
void SaveSettProc(void){
	if (config.save > 0){
		config.save--;
		if (config.save == 0){
			cli();
			SEG_PORT = 0xFF;
			ee_write(EE_ADDRESS, config.time);
			_delay_ms(50);
			sei();
		}
	}
}

//-----------------------------------------------------------
void RestoreSett(void){
	config.time = ee_read(EE_ADDRESS);
	if (config.time > TIME_MAX) {
		config.time = TIME_DEFAULT;
	}
}